#include "searchtimer_thread.h"
#include "epgsearchcfg.h"
#include "tools.h"
#include <vdr/tools.h>
#include <vdr/plugin.h>

extern void ToLower(char* szText);

void sleepMSec(long ms)
{
#if VDRVERSNUM >= 10314
    cCondWait::SleepMs(ms);
#else
    usleep(1000 * ms);
#endif
}

void sleepSec(long s)
{
    sleepMSec(s * 1000);
}

cSearchTimerThread *cSearchTimerThread::m_Instance = NULL;

cSearchTimerThread::cSearchTimerThread(void)
: cThread("EPGSearch: searchtimer")
{
    m_Active = false;
    m_lastUpdate = time(NULL);
}

cSearchTimerThread::~cSearchTimerThread() {
    if (m_Active) 
	Stop();
}

void cSearchTimerThread::Init(void) {
    if (!EPGSearchConfig.useSearchTimers)
	return;
    if (m_Instance == NULL) {
	m_Instance = new cSearchTimerThread;
	m_Instance->Start();
    }
}

void cSearchTimerThread::Exit(void) {
    if (m_Instance != NULL) {
	m_Instance->Stop();
	DELETENULL(m_Instance);
    }
}

void cSearchTimerThread::Stop(void) {
    m_Active = false;
    Cancel(3);
}

void cSearchTimerThread::Action(void) 
{
    if (!SVDRPSendCmd)
    {
	esyslog("EPGSearch: ERROR - SVDRPSend script not specified or does not exist(use -f option)");
	return;
    }
    m_Active = true;
    // let VDR do its startup
    sleepSec(15);
    time_t nextUpdate = time(NULL);
    while (m_Active) 
    {
	time_t now = time(NULL);
	if (now >= nextUpdate || NeedUpdate())
	{
	    if (Timers.BeingEdited()) 		
	    {
		sleepSec(1);
		continue;
	    }
	    isyslog("EPGSearch: search timer update started");

	    cSearchExt *searchExt = SearchExts.First();
	    while (searchExt) {
		if (!searchExt->useAsSearchTimer)
		{
		    searchExt = SearchExts.Next(searchExt);
		    continue;
		}

		sleepMSec(1);
		cSchedulesLock schedulesLock;
		const cSchedules *schedules;
		schedules = cSchedules::Schedules(schedulesLock);
		if(!schedules) {
		    esyslog("cSearchTimerThread::Action scheduler is empty!");
		    continue;
		}

		const cSchedule *Schedule = schedules->First();
		const cEvent *pPrevEvent = NULL;
		const cEvent *pEvent = NULL;

		while (Schedule) {
		    if (searchExt->useChannel && searchExt->channelMin && searchExt->channelMax)
		    {
			cChannel* channel = Channels.GetByChannelID(Schedule->ChannelID(),true);
			if (channel && (searchExt->channelMin->Number() > channel->Number() || searchExt->channelMax->Number() < channel->Number()))
			{
			    Schedule = (const cSchedule *)schedules->Next(Schedule);
			    continue;
			}
		    }

		    pPrevEvent = NULL;
		    do {
			pEvent = searchExt->GetEventBySearchExt(Schedule, pPrevEvent);
			pPrevEvent = pEvent;
			if (pEvent) {

			    cChannel *channel = Channels.GetByChannelID(pEvent->ChannelID(), true);
			    if (channel) {
				int index = 0;
				cTimer *timer = new cTimer(pEvent);

				if (searchExt->useEpisode)
				{
				    char* pFile = NULL;
				    if (!isempty(pEvent->ShortText()))
					asprintf(&pFile, "%s~%s", pEvent->Title(), pEvent->ShortText());
				    else
					asprintf(&pFile, "%s~%s-%s", pEvent->Title(), GETDATESTRING(pEvent), GETTIMESTRING(pEvent));
				    pFile = strreplace(pFile, ':', '|');
				    pFile = strreplace(pFile, " ~", "~");
				    pFile = strreplace(pFile, "~ ", "~");
				    timer->SetFile(pFile);
				    free(pFile);
				}
				if (!isempty(searchExt->directory))
				{
				    char* pFile = NULL;
				    asprintf(&pFile, "%s~%s", searchExt->directory, timer->File());
				    pFile = strreplace(pFile, ':', '|');
				    pFile = strreplace(pFile, " ~", "~");
				    pFile = strreplace(pFile, "~ ", "~");
				    timer->SetFile(pFile);
				    free(pFile);
				}

				cTimer *t = Timers.GetTimer(timer);
				if (t) 
				{ // already exists
				    if (strcmp(t->File(), timer->File()) == 0)
				    { // dir, title and episode name have not changed
					delete timer;
					continue;
				    }
				    else
					index = t->Index()+1;
				}
				else
				{
				    // quite week test:
				    // search for a timer that differs max. 10 min from start and end of the event
				    time_t eStart = pEvent->StartTime();
				    time_t eStop = pEvent->EndTime();
				    for (cTimer *ti = Timers.First(); ti; ti = Timers.Next(ti)) 
				    {
					if (channel != ti->Channel())
					    continue;

					struct tm tm_r;
					time_t etmStart = pEvent->StartTime();
					tm *tm = localtime_r(&etmStart, &tm_r);
					if (tm->tm_mday != ti->Day())
					    continue;

					time_t tStart = ti->StartTime() + Setup.MarginStart * 60;
					time_t tStop = ti->StopTime() - Setup.MarginStop * 60; 
					if (abs(tStart - eStart) < 10 * 60 && abs(tStop - eStop) < 10 * 60)
					{
					    index = ti->Index()+1;
					    break;
					}
				    }

				}

				AddTimer(timer, index);
				delete timer;
				if (index == 0)
				    isyslog("EPGSearch: added timer for %s (%s - %s)", pEvent->Title(), GETDATESTRING(pEvent), GETTIMESTRING(pEvent));
				else
				    isyslog("EPGSearch: modified timer %d for %s (%s - %s)", index, pEvent->Title(), GETDATESTRING(pEvent), GETTIMESTRING(pEvent));
			    }
			}
		    } while(pPrevEvent);
		    Schedule = (const cSchedule *)schedules->Next(Schedule);
		}

		searchExt = SearchExts.Next(searchExt);
	    }
	    m_lastUpdate = time(NULL);
	    nextUpdate = m_lastUpdate + (EPGSearchConfig.UpdateIntervall * 60);
	    dsyslog("EPGSearch: search timer update finished");
	}
	sleepSec(5);
    };
    dsyslog("EPGSearch: Leaving search timer thread");
}

bool cSearchTimerThread::NeedUpdate()
{
    return  m_lastUpdate <= LastModifiedTime(AddDirectory( cPlugin::ConfigDirectory(), ".epgsearchupdate"));
}

void cSearchTimerThread::AddTimer(cTimer* Timer, int index) 
{
    char *cmdbuf = NULL;
    struct tm tm_r;
    static char bufStart[25];
    time_t start = Timer->StartTime();
    strftime(bufStart, sizeof(bufStart), "%H%M", localtime_r(&start, &tm_r));
    static char bufEnd[25];
    time_t stop = Timer->StopTime();
    strftime(bufEnd, sizeof(bufEnd), "%H%M", localtime_r(&stop, &tm_r));

    if (index==0)
	asprintf(&cmdbuf, "%s -p %d \"NEWT 1:%d:%d:%s:%s:%d:%d:%s:\"", 
		 SVDRPSendCmd,
		 EPGSearchConfig.SVDRPPort,
		 Timer->Channel()->Number(),
		 Timer->Day(),
		 bufStart,
		 bufEnd,
		 Timer->Priority(),
		 Timer->Lifetime(),
		 Timer->File());
    else
	asprintf(&cmdbuf, "%s -p %d \"MODT %d 1:%d:%d:%s:%s:%d:%d:%s:\"", 
		 SVDRPSendCmd,
		 EPGSearchConfig.SVDRPPort,
		 index,
		 Timer->Channel()->Number(),
		 Timer->Day(),
		 bufStart,
		 bufEnd,
		 Timer->Priority(),
		 Timer->Lifetime(),
		 Timer->File());

    FILE *p = popen(cmdbuf, "r");
    if (p)
	pclose(p);
    else
	esyslog("ERROR: can't open pipe for command '%s'", cmdbuf);
    free(cmdbuf);
}
